#!/usr/bin/env python
#
# Copyright (c) 2011, 2012
# Contributors to the Freedoom project.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the freedoom project nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ----------------------------------------------------------------------
#
# Module to load OPL instrument files.
#

import os
import sys
import sbi_file
import a2i_file

# Check the specified instrument data is OPL2-compatible and does not
# use any OPL3 features. Returns an error message, or 'None' if data
# is valid.

def check_opl2(filename, data):
	def opl2_warning(message):
		print >> sys.stderr, "%s: %s" % (filename, message)

	# CHA,B control stereo, but are ignored on OPL2, so it's no problem:
	#if (data["feedback_fm"] & 0xf0) != 0:
	#	opl2_warning("Cannot use CHA,B,C,D: %02x" % data["feedback_fm"])

	if data["m_waveform"] > 3:
		opl2_warning("Modulator uses waveform %i: only 0-3 supported" %
		             data["m_waveform"])
	if data["c_waveform"] > 3:
		opl2_warning("Carrier uses waveform %i: only 0-3 supported" %
		             data["c_waveform"])

def load_instrument(filename):

	# As a hack, a literal dictionary of the values can be specified
	# in place of a filename.

	if isinstance(filename, dict):
		return filename

	filename = os.path.join("instruments", filename)

	if filename.endswith(".a2i"):
		result = a2i_file.read(filename)
	elif filename.endswith(".sbi"):
		result = sbi_file.read(filename)
	else:
		raise Exception("Unknown instrument file type: '%s'" % filename)

	check_opl2(filename, result)

	return result

class Instrument:
	def __init__(self, file1, file2=None, off1=0, off2=0, note=None):
		self.voice1 = load_instrument(file1)

		if file2 is not None:
			self.voice2 = load_instrument(file2)
		else:
			self.voice2 = None

		self.fixed_note = note
		self.offset1 = off1
		self.offset2 = off2

NullInstrument = Instrument("dummy.sbi")

if __name__ == "__main__":
	for filename in sys.argv[1:]:
		Instrument(filename)




#!/usr/bin/env python
#
# Copyright (c) 2011, 2012
# Contributors to the Freedoom project.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the freedoom project nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ----------------------------------------------------------------------
#
# Constants for MIDI notes.
#
# For example:
#    F# in Octave 3:      O3.Fs
#    C# in Octave -2:     On2.Cs
#    D-flat in Octave 1:  O1.Db
#    D in Octave 0:       O0.D
#    E in Octave 2:       O2.E

class Octave:
	def __init__(self, base):
		self.C  = base
		self.Cs = base + 1
		self.Db = base + 1
		self.D  = base + 2
		self.Ds = base + 3
		self.Eb = base + 3
		self.E  = base + 4
		self.F  = base + 5
		self.Fs = base + 6
		self.Gb = base + 6
		self.G  = base + 7
		self.Gs = base + 8
		self.Ab = base + 8
		self.A  = base + 9
		self.As = base + 10
		self.Bb = base + 10
		self.B  = base + 11

On5 = Octave(0)      # Octave -5
On4 = Octave(12)     # Octave -4
On3 = Octave(24)     # Octave -3
On2 = Octave(36)     # Octave -2
On1 = Octave(48)     # Octave -1
O0  = Octave(60)     # Octave 0
O1  = Octave(72)     # Octave 1
O2  = Octave(84)     # Octave 2
O3  = Octave(96)     # Octave 3
O4  = Octave(108)    # Octave 4
O5  = Octave(120)    # Octave 5

# Given a MIDI note number, return a note definition in terms of the
# constants above.

def def_for_note(note):
	OCTAVES = [ "On5", "On4", "On3", "On2", "On1",
	            "O0", "O1", "O2", "O3", "O4", "O5" ]
	NOTES = [ "C", "Cs", "D", "Ds", "E", "F", "Fs",
	          "G", "Gs", "A", "As", "B" ]

	return "%s.%s" % (OCTAVES[note / 12], NOTES[note % 12])




"""A module for manipulating Doom data structures."""

class Patch:
	def __init__(self, n,x,y):
		self.name = n
		self.yoff = x
		self.xoff =y

	def __str__(self):
		return "*\t%8s\t\t%d\t%d" % (self.name,self.xoff,self.yoff)

class Texture:
	def __init__(self,name,width,height):
		self.name = name
		self.width = width
		self.height = height
		self.patches = []
		self.pixbuf = None

	def __str__(self):
		me   = "%8s\t\t%d\t%d\n" % (self.name,int(self.width),int(self.height))
		kids = "\n".join(map(str, self.patches))
		if kids:
			kids += "\n"
		return (me + kids)




#!/usr/bin/env python

# example images.py

import pygtk
pygtk.require('2.0')
import gtk

class ImagesExample:
    # when invoked (via signal delete_event), terminates the application.
    def close_application(self, widget, event, data=None):
        gtk.main_quit()
        return False

    def __init__(self):
        # create the main window, and attach delete_event signal to terminating
        # the application
        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.connect("delete_event", self.close_application)
        window.set_border_width(10)
        window.show()

        # a horizontal box to hold the buttons
        hbox = gtk.HBox()
        hbox.show()
        window.add(hbox)

        # create several images with data from files and load images into
        # buttons
        baseimage = gtk.Image()
        baseimage.set_from_file("../../patches/wall40_1.gif")
        pixbuf = baseimage.get_pixbuf()
        if pixbuf:
            scale = 1
            baseimage.set_from_pixbuf(pixbuf.scale_simple(
                pixbuf.get_width()  * scale,
                pixbuf.get_height() * scale,
                gtk.gdk.INTERP_NEAREST
            ))
        baseimage.show()

        basepatch = gtk.Image()
        basepatch.set_from_file("../../patches/ps20a0.gif")
        pb = basepatch.get_pixbuf()
        basepatch.set_from_pixbuf(pb.scale_simple(
            pb.get_width()  * scale,
            pb.get_height() * scale,
            gtk.gdk.INTERP_NEAREST
        ))
        pb = basepatch.get_pixbuf()
        pb = pb.add_alpha(True,chr(0),chr(255),chr(255))

        for x,y in [(0,0), (24,24), (104,16), (24,104), (-16,0), (0,-16)]:
            x *= scale
            y *= scale
            image = gtk.Image()
            image.set_from_pixbuf(baseimage.get_pixbuf().copy())
            image.show()

            dest_x = max(0, x)
            dest_y = max(0, y)
            dest_height = min(pb.get_height(), image.get_pixbuf().get_height() - dest_y)
            dest_width  = min(pb.get_width(),  image.get_pixbuf().get_width()  - dest_x)
            if x < 0:
                dest_width += x
            if y < 0:
                dest_height += y
            offset_x = x
            offset_y = y

            pb.composite(
                image.get_pixbuf(),
                dest_x, dest_y, dest_width, dest_height,
                offset_x, offset_y, 1, 1, # scale
                gtk.gdk.INTERP_NEAREST, 255
            )

            # a button to contain the image widget
            button = gtk.Button()
            button.add(image)
            button.show()
            hbox.pack_start(button)

def main():
    gtk.main()
    return 0

if __name__ == "__main__":
    ImagesExample()
    main()



#!/usr/bin/env python
#
# Copyright (c) 2011, 2012
# Contributors to the Freedoom project.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the freedoom project nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ----------------------------------------------------------------------
#
# Module to load A2I (AdlibTracker 2) instrument files.
#

import sys
import struct

HEADER_STRING = "_A2ins_"

class BitReader:
	def __init__(self, data):
		self.data = data
		self.index = 0
		self.byte = 0
		self.byte_bit = 0

	def read_byte(self):
		if self.index >= len(self.data):
			raise IndexError("Reached end of decompress stream " +
			                 "(%i bytes)" % len(self.data))
		result, = struct.unpack("B", self.data[self.index:self.index+1])
		self.index += 1
		return result

	def read_bit(self):
		if self.byte_bit <= 0:
			self.byte = self.read_byte()
			self.byte_bit = 7
		else:
			self.byte_bit -= 1

		if (self.byte & (1 << self.byte_bit)) != 0:
			result = 1
		else:
			result = 0

		return result

	def read_bits(self, n):
		result = 0

		for i in range(n):
			result = (result << 1) + self.read_bit()

		return result

def read_gamma(reader):
	result = 1

	while True:
		result = (result << 1) | reader.read_bit()

		if reader.read_bit() == 0:
			break

	return result

def decompress(data, data_len):
	reader = BitReader(data)
	result = []
	lwm = 0
	last_offset = 0

	# First byte is an implied straight copy.
	result.append(reader.read_byte())

	while True:
		if reader.read_bit():
			if reader.read_bit():
				if reader.read_bit():
					# 111 = Copy byte from history,
					# up to 15 bytes back.
					#print "111 copy"

					offset = reader.read_bits(4)

					if offset == 0:
						result.append(0)
					else:
						b = result[len(result) - offset]
						result.append(b)

					lwm = 0
				else:
					#print "110 copy"
					# 110 = Copy 2-3 bytes from
					# further back in history

					offset = reader.read_byte()
					count = 2 + (offset & 0x01)
					offset = offset >> 1

					if offset == 0:
						break

					index = len(result) - offset
					for i in range(count):
						result += result[index:index+1]
						index += 1

					last_offset = offset
					lwm = 1
			else:
				# 10 = Copy from further away...

				offset = read_gamma(reader)

				if lwm == 0 and offset == 2:
					#print "10 copy type 1"
					count = read_gamma(reader)
					index = len(result) - last_offset
					for i in range(count):
						result += result[index:index+1]
						index += 1
				else:
					#print "10 copy type 2"
					if lwm == 0:
						offset -= 3
					else:
						offset -= 2

					offset = offset * 256 \
					       + reader.read_byte()

					count = read_gamma(reader)

					if offset >= 32000:
						count += 1
					if offset >= 1280:
						count += 1
					if offset < 128:
						count += 2

					index = len(result) - offset
					for i in range(count):
						result += result[index:index+1]
						index += 1

					last_offset = offset

				lwm = 1

		else:
			#print "Single byte output"

			# 0 = Straight-through byte copy.
			result.append(reader.read_byte())
			lwm = 0

		#print "len: %i" % len(result)

	return struct.pack("%iB" % len(result), *result)

FIELDS = [
	"m_am_vibrato_eg",
	"c_am_vibrato_eg",
	"m_ksl_volume",
	"c_ksl_volume",
	"m_attack_decay",
	"c_attack_decay",
	"m_sustain_release",
	"c_sustain_release",
	"m_waveform",
	"c_waveform",
	"feedback_fm",
	"panning",
	"finetune",
	"voice_type",
]

def decode_type_9(data):
	compressed_len, = struct.unpack("<H", data[0:2])
	compressed_data = data[2:2+compressed_len]
	decompressed_data = decompress(compressed_data, compressed_len)

	instr_data = {}

	# Decode main fields:

	for i in range(len(FIELDS)):
		instr_data[FIELDS[i]], = \
		    struct.unpack("B", decompressed_data[i:i+1])

	# Decode instrument name

	ps = decompressed_data[14:]
	instr_data["name"], = struct.unpack("%ip" % len(ps), ps)

	return instr_data

def read(filename):
	f = open(filename)
	data = f.read()
	f.close()

	hdrstr, crc, filever = struct.unpack("<7sHB", data[0:10])

	if hdrstr.lower() != HEADER_STRING.lower():
		raise Exception("Wrong file header ID string")

	# TODO: CRC?

	if filever == 9:
		return decode_type_9(data[10:])
	else:
		raise Exception("Unsupported file version: %i" % filever)

if __name__ == "__main__":
	for filename in sys.argv[1:]:
		print filename
		print read(filename)




#!/usr/bin/env python
#
# Copyright (C) 2001 Colin Phipps <cphipps@doomworld.com>
# Copyright (C) 2008, 2013 Simon Howard
# Parts copyright (C) 1999 by id Software (http://www.idsoftware.com/)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# Takes PLAYPAL as input (filename is the only parameter)
# Produces a light graduated COLORMAP on stdout
# O(n^2)
#
# This was originally a Perl script by Colin Phipps; it was converted
# to Python and now is a more generic tool for generating all kinds of
# COLORMAP effects.
#

import os
import sys
import struct

# Parameters affecting colormap generation:

# "Darkness" is this color, which is usually black, but can be
# overridden (RGB 0-255):
dark_color = (0, 0, 0)

# Color to tint the colormap (RGB 0.0-1.0):
tint_color = (255, 255, 255)

# Fractional balance between tint and normal color. 0 is no tint applied,
# 1.0 is full tint.
tint_frac = 0

# Fudge factor to adjust brightness when calculating 'tinted' version
# of colors. Larger values are brighter but may cause color clipping.
# A value of 0.33 is a straight-average of the RGB channels. Maximum
# sensible value is 1.0, though it can be overdriven for fancy
# brightness effects.
tint_bright = 0.5

def read_palette(filename):
	"""Read palette from file and return a list of tuples containing
	   RGB values."""
	f = open(filename, "rb")

	colors = []

	for i in range(256):
		data = f.read(3)

		color = struct.unpack("BBB", data)
		colors.append(color)

	return colors

# Return closest palette entry to the given RGB triple

def search_palette(palette, target):
	"""Search the given palette and find the nearest matching
	   color to the given color, returning an index into the
	   palette of the color that best matches."""
	best_diff = None
	best_index = None

	def square(x):
		return x * x

	for i in range(len(palette)):
		color = palette[i]

		diff = square(target[0] - color[0])            \
		     + square(target[1] - color[1])            \
		     + square(target[2] - color[2])

		if best_index is None or diff < best_diff:
			best_diff = diff
			best_index = i

	return best_index

def generate_colormap(colors, palette):
	"""Given a list of colors, translate these into indexes into
	   the given palette, finding the nearest color where an exact
	   match cannot be found."""
	result = []

	for color in colors:
		index = search_palette(palette, color)
		result.append(index)

	return result

def tint_colors(colors, tint, bright=0.5):
	"""Given a list of colors, tint them a particular color."""

	result = []
	for c in colors:
		# I've experimented with different methods of calculating
		# intensity, but this seems to work the best. This is basically
		# doing an average of the full channels, but a straight
		# average causes the picture to get darker - eg. (0,0,255)
		# maps to (87,87,87). So we have a controllable brightness
		# factor that allows the brightness to be adjusted.
		intensity = min((c[0] + c[1] + c[2]) * bright, 255) / 255.0
		result.append((
			tint[0] * intensity,
			tint[1] * intensity,
			tint[2] * intensity,
		))

	return result

def blend_colors(colors1, colors2, factor=0.5):
	"""Blend the two given lists of colors, with 'factor' controlling
	   the mix between the two. factor=0 is exactly colors1, while
	   factor=1 is exactly colors2. Returns a list of blended colors."""
	result = []

	for index, c1 in enumerate(colors1):
		c2 = colors2[index]

		result.append((
			c2[0] * factor + c1[0] * (1 - factor),
			c2[1] * factor + c1[1] * (1 - factor),
			c2[2] * factor + c1[2] * (1 - factor),
		))

	return result

def invert_colors(colors):
	"""Given a list of colors, translate them to inverted monochrome."""
	result = []

	for color in colors:
		average = (color[0] + color[1] + color[2]) / 3
		inverse = 255 - average

		result.append((inverse, inverse, inverse))

	return result

def solid_color_list(color):
	"""Generate a 256-entry palette where all entries are the
	   same color."""
	return [color] * 256

def output_colormap(colormap):
	"""Output the given palette to stdout."""
	for c in colormap:
		x = struct.pack("B", c)
		os.write(sys.stdout.fileno(), x)

def print_palette(colors):
	for y in range(16):
		for x in range(16):
			color = colors[y * 16 + x]

			print("#%02x%02x%02x" % color)

		print()

def parse_color_code(s):
	"""Parse a color code in HTML color code format, into an RGB
	   3-tuple value."""
	if not s.startswith('#') or len(s) != 7:
		raise Exception('Not in HTML color code form: %s' % s)
	return (int(s[1:3], 16), int(s[3:5], 16), int(s[5:7], 16))

def set_parameter(name, value):
	"""Set configuration value, from command line parameters."""
	global dark_color, tint_color, tint_frac, tint_bright

	if name == 'dark_color':
		dark_color = parse_color_code(value)
	elif name == 'tint_color':
		tint_color = parse_color_code(value)
	elif name == 'tint_pct':
		tint_frac = int(value) / 100.0
	elif name == 'tint_bright':
		tint_bright = float(value)
	else:
		raise Exception("Unknown parameter: '%s'" % name)

# Parse command line.

playpal_filename = None

for arg in sys.argv[1:]:
	if arg.startswith('--') and '=' in arg:
		key, val = arg[2:].split('=', 2)
		set_parameter(key, val)
	else:
		playpal_filename = arg

if playpal_filename is None:
	print("Usage: %s playpal.lmp > output-file.lmp" % sys.argv[0])
	sys.exit(1)

palette = read_palette(playpal_filename)
colors = palette

# Apply tint, if enabled.
# The tint is intentionally applied *before* the darkening effect is
# applied. This allows us to darken to a different color than the tint
# color, if so desired.
if tint_frac > 0:
	colors = blend_colors(palette,
	                      tint_colors(colors, tint_color, tint_bright),
	                      tint_frac)

# Generate colormaps for different darkness levels, by blending between
# the default colors and a palette where every entry is the "dark" color.
dark = solid_color_list(dark_color)

for i in range(32):
	darken_factor = (32 - i) / 32.0
	darkened_colors = blend_colors(dark, colors, darken_factor)
	output_colormap(generate_colormap(darkened_colors, palette))

# Inverse color map for invulnerability effect.
inverse_colors = invert_colors(palette)
output_colormap(generate_colormap(inverse_colors, palette))




#!/usr/bin/env python
#
# Copyright (c) 2013 Contributors to the Freedoom project.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the freedoom project nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ----------------------------------------------------------------------
#
# Script to gather statistics from MIDI files about instrument usage.
# Generates stats.py which is used as input for the ultramid.ini
# generator script.
#
# This script requires python-midi; see here:
#   https://github.com/vishnubob/python-midi
#

import midi
import sys

def get_instr_stats(filename):
	"""Get a set of instruments used by the specified MIDI file."""
	result = set()
	midfile = midi.read_midifile(filename)

	for track in midfile:
		for event in track:
			if isinstance(event, midi.ProgramChangeEvent) \
			   and event.channel != 9:
				instr = event.data[0]
				result.add(instr)
			# Percussion:
			if isinstance(event, midi.NoteOnEvent) \
			   and event.channel == 9:
				instr = event.data[0] + 128
				result.add(instr)

	return result

total_stats = [0] * 217

for filename in sys.argv[1:]:
	print "Processing %s" % filename
	stats = get_instr_stats(filename)
	print sorted(stats)
	for instrument in stats:
		total_stats[instrument] += 1

with open("stats.py", "w") as f:
	f.write("# Instrument stats, autogenerated by gather_stats.py\n\n")
	f.write("INSTRUMENT_STATS = [\n\t")

	for index, stat in enumerate(total_stats):
		f.write("% 5i," % stat)
		if (index % 10) == 9:
			f.write("\n\t")

	f.write("\n]\n")





#!/usr/bin/env python
#
# Copyright (c) 2011, 2012
# Contributors to the Freedoom project.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the freedoom project nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ----------------------------------------------------------------------
#
# Module for interacting with Doom GENMIDI lumps.
#

from instrument import Instrument, NullInstrument
import struct
import sys

GENMIDI_HEADER = "#OPL_II#"
NUM_INSTRUMENTS = 175
INSTR_DATA_LEN = 36
INSTR_NAME_LEN = 32

FLAG_FIXED_PITCH = 0x0001
FLAG_TWO_VOICE   = 0x0004

KSL_MASK = 0xc0
VOLUME_MASK = 0x3f

# Order of fields in GENMIDI data structures.

GENMIDI_FIELDS = [
	"m_am_vibrato_eg",
	"m_attack_decay",
	"m_sustain_release",
	"m_waveform",
	"m_ksl",
	"m_volume",
	"feedback_fm",
	"c_am_vibrato_eg",
	"c_attack_decay",
	"c_sustain_release",
	"c_waveform",
	"c_ksl",
	"c_volume",
	"null",
	"note_offset"
]

# Encode a single voice of an instrument to binary.

def encode_voice(data, offset):
	result = dict(data)

	result["m_ksl"] = data["m_ksl_volume"] & KSL_MASK
	result["m_volume"] = data["m_ksl_volume"] & VOLUME_MASK
	result["c_ksl"] = data["c_ksl_volume"] & KSL_MASK
	result["c_volume"] = data["c_ksl_volume"] & VOLUME_MASK

	result["null"] = 0
	result["note_offset"] = offset

	return struct.pack("<BBBBBBBBBBBBBBh",
	                   *map(lambda key: result[key], GENMIDI_FIELDS))

# Encode an instrument to binary.

def encode_instrument(instrument):
	flags = 0

	instr1_data = encode_voice(instrument.voice1, instrument.offset1)

	if instrument.voice2 is not None:
		flags |= FLAG_TWO_VOICE
		instr2_data = encode_voice(instrument.voice2,
		                           instrument.offset2)
	else:
		instr2_data = encode_voice(NullInstrument.voice1, 0)

	if instrument.fixed_note is not None:
		flags |= FLAG_FIXED_PITCH
		fixed_note = instrument.fixed_note
	else:
		fixed_note = 0

	header = struct.pack("<hBB", flags, 128, fixed_note)

	return header + instr1_data + instr2_data

def encode_instruments(instruments):
	result = []

	for instrument in instruments:
		result.append(encode_instrument(instrument))

	return b"".join(result)

def encode_instrument_names(instruments):
	result = []

	for instrument in instruments:
		result.append(struct.pack("32s", instrument.voice1["name"]))

	return b"".join(result)

def write(filename, instruments):
	header = struct.pack("%is" % len(GENMIDI_HEADER), GENMIDI_HEADER)

	f = open(filename, 'w')

	f.write(header)
	f.write(encode_instruments(instruments))
	f.write(encode_instrument_names(instruments))

	f.close()

def decode_voice(data, name):

	fields = struct.unpack("<BBBBBBBBBBBBBBh", data)

	result = {}
	for i in range(len(GENMIDI_FIELDS)):
		result[GENMIDI_FIELDS[i]] = fields[i]

	result["m_ksl_volume"] = result["m_ksl"] | result["m_volume"]
	result["c_ksl_volume"] = result["c_ksl"] | result["c_volume"]
	result["name"] = name.rstrip("\0")

	return result

def decode_instrument(data, name):
	flags, finetune, fixed_note = struct.unpack("<hBB", data[0:4])

	voice1 = decode_voice(data[4:20], name)
	offset1 = voice1["note_offset"]

	# Second voice?

	if (flags & FLAG_TWO_VOICE) != 0:
		voice2 = decode_voice(data[20:], name)
		offset2 = voice2["note_offset"]
	else:
		voice2 = None
		offset2 = 0

	# Null out fixed_note if the fixed pitch flag isn't set:

	if (flags & FLAG_FIXED_PITCH) == 0:
		fixed_note = None

	return Instrument(voice1, voice2,
	                  off1=offset1, off2=offset2,
	                  note=fixed_note)

def read(filename):
	f = open(filename)
	data = f.read()
	f.close()

	# Check header:

	header = data[0:len(GENMIDI_HEADER)]
	if header != GENMIDI_HEADER:
		raise Exception("Incorrect header for GENMIDI lump")

	body = data[len(GENMIDI_HEADER):]
	instr_data = body[0:NUM_INSTRUMENTS * INSTR_DATA_LEN]
	instr_names = body[NUM_INSTRUMENTS * INSTR_DATA_LEN:]
	result = []

	for i in range(NUM_INSTRUMENTS):
		data = instr_data[i * INSTR_DATA_LEN:(i+1) * INSTR_DATA_LEN]
		name = instr_names[i * INSTR_NAME_LEN:(i+1) * INSTR_NAME_LEN]

		result.append(decode_instrument(data, name))

	return result

if __name__ == "__main__":
	for filename in sys.argv[1:]:
		instruments = read(filename)
		for i in range(len(instruments)):
			instrument = instruments[i]
			fixed_note = instrument.fixed_note

			if fixed_note is not None:
				print "%i (fixed note: %i):" % (i, fixed_note)
			else:
				print "%i:" % i

			print "\tVoice 1: %s" % instrument.voice1
			if instrument.voice2 is not None:
				print "\tVoice 2: %s" % instrument.voice2




#
# Copyright (c) 2013 Contributors to the Freedoom project.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the freedoom project nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# --------------------------------------------------------------
#
# Config file for GUS config generator.
#

# Names of GUS instrument files to use for playing MIDI.
# These are the names of the .pat files in the \ultrasnd\midi directory
# that are loaded into the card.

GUS_INSTR_PATCHES = {
	  0: "acpiano",    # #001 - Acoustic Grand Piano
	  1: "britepno",   # #002 - Bright Acoustic Piano
	  2: "synpiano",   # #003 - Electric Grand Piano
	  3: "honky",      # #004 - Honky-tonk Piano
	  4: "epiano1",    # #005 - Electric Piano 1
	  5: "epiano2",    # #006 - Electric Piano 2
	  6: "hrpschrd",   # #007 - Harpsichord
	  7: "clavinet",   # #008 - Clavi
	  8: "celeste",    # #009 - Celesta
	  9: "glocken",    # #010 - Glockenspiel
	 10: "musicbox",   # #011 - Music Box
	 11: "vibes",      # #012 - Vibraphone
	 12: "marimba",    # #013 - Marimba
	 13: "xylophon",   # #014 - Xylophone
	 14: "tubebell",   # #015 - Tubular Bells
	 15: "santur",     # #016 - Dulcimer
	 16: "homeorg",    # #017 - Drawbar Organ
	 17: "percorg",    # #018 - Percussive Organ
	 18: "rockorg",    # #019 - Rock Organ
	 19: "church",     # #020 - Church Organ
	 20: "reedorg",    # #021 - Reed Organ
	 21: "accordn",    # #022 - Accordion
	 22: "harmonca",   # #023 - Harmonica
	 23: "concrtna",   # #024 - Tango Accordion
	 24: "nyguitar",   # #025 - Acoustic Guitar (nylon)
	 25: "acguitar",   # #026 - Acoustic Guitar (steel)
	 26: "jazzgtr",    # #027 - Electric Guitar (jazz)
	 27: "cleangtr",   # #028 - Electric Guitar (clean)
	 28: "mutegtr",    # #029 - Electric Guitar (muted)
	 29: "odguitar",   # #030 - Overdriven Guitar
	 30: "distgtr",    # #031 - Distortion Guitar
	 31: "gtrharm",    # #032 - Guitar harmonics
	 32: "acbass",     # #033 - Acoustic Bass
	 33: "fngrbass",   # #034 - Electric Bass (finger)
	 34: "pickbass",   # #035 - Electric Bass (pick)
	 35: "fretless",   # #036 - Fretless Bass
	 36: "slapbas1",   # #037 - Slap Bass 1
	 37: "slapbas2",   # #038 - Slap Bass 2
	 38: "synbass1",   # #039 - Synth Bass 1
	 39: "synbass2",   # #040 - Synth Bass 2
	 40: "violin",     # #041 - Violin
	 41: "viola",      # #042 - Viola
	 42: "cello",      # #043 - Cello
	 43: "contraba",   # #044 - Contrabass
	 44: "tremstr",    # #045 - Tremolo Strings
	 45: "pizzcato",   # #046 - Pizzicato Strings
	 46: "harp",       # #047 - Orchestral Harp
	 47: "timpani",    # #048 - Timpani
	 48: "marcato",    # #049 - String Ensemble 1
	 49: "slowstr",    # #050 - String Ensemble 2
	 50: "synstr1",    # #051 - SynthStrings 1
	 51: "synstr2",    # #052 - SynthStrings 2
	 52: "choir",      # #053 - Choir Aahs
	 53: "doo",        # #054 - Voice Oohs
	 54: "voices",     # #055 - Synth Voice
	 55: "orchhit",    # #056 - Orchestra Hit
	 56: "trumpet",    # #057 - Trumpet
	 57: "trombone",   # #058 - Trombone
	 58: "tuba",       # #059 - Tuba
	 59: "mutetrum",   # #060 - Muted Trumpet
	 60: "frenchrn",   # #061 - French Horn
	 61: "hitbrass",   # #062 - Brass Section
	 62: "synbras1",   # #063 - SynthBrass 1
	 63: "synbras2",   # #064 - SynthBrass 2
	 64: "sprnosax",   # #065 - Soprano Sax
	 65: "altosax",    # #066 - Alto Sax
	 66: "tenorsax",   # #067 - Tenor Sax
	 67: "barisax",    # #068 - Baritone Sax
	 68: "oboe",       # #069 - Oboe
	 69: "englhorn",   # #070 - English Horn
	 70: "bassoon",    # #071 - Bassoon
	 71: "clarinet",   # #072 - Clarinet
	 72: "piccolo",    # #073 - Piccolo
	 73: "flute",      # #074 - Flute
	 74: "recorder",   # #075 - Recorder
	 75: "woodflut",   # #076 - Pan Flute
	 76: "bottle",     # #077 - Blown Bottle
	 77: "shakazul",   # #078 - Shakuhachi
	 78: "whistle",    # #079 - Whistle
	 79: "ocarina",    # #080 - Ocarina
	 80: "sqrwave",    # #081 - Lead 1 (square)
	 81: "sawwave",    # #082 - Lead 2 (sawtooth)
	 82: "calliope",   # #083 - Lead 3 (calliope)
	 83: "chiflead",   # #084 - Lead 4 (chiff)
	 84: "charang",    # #085 - Lead 5 (charang)
	 85: "voxlead",    # #086 - Lead 6 (voice)
	 86: "lead5th",    # #087 - Lead 7 (fifths)
	 87: "basslead",   # #088 - Lead 8 (bass + lead)
	 88: "fantasia",   # #089 - Pad 1 (new age)
	 89: "warmpad",    # #090 - Pad 2 (warm)
	 90: "polysyn",    # #091 - Pad 3 (polysynth)
	 91: "ghostie",    # #092 - Pad 4 (choir)
	 92: "bowglass",   # #093 - Pad 5 (bowed)
	 93: "metalpad",   # #094 - Pad 6 (metallic)
	 94: "halopad",    # #095 - Pad 7 (halo)
	 95: "sweeper",    # #096 - Pad 8 (sweep)
	 96: "aurora",     # #097 - FX 1 (rain)
	 97: "soundtrk",   # #098 - FX 2 (soundtrack)
	 98: "crystal",    # #099 - FX 3 (crystal)
	 99: "atmosphr",   # #100 - FX 4 (atmosphere)
	100: "freshair",   # #101 - FX 5 (brightness)
	101: "unicorn",    # #102 - FX 6 (goblins)
	102: "echovox",    # #103 - FX 7 (echoes)
	103: "startrak",   # #104 - FX 8 (sci-fi)
	104: "sitar",      # #105 - Sitar
	105: "banjo",      # #106 - Banjo
	106: "shamisen",   # #107 - Shamisen
	107: "koto",       # #108 - Koto
	108: "kalimba",    # #109 - Kalimba
	109: "bagpipes",   # #110 - Bag pipe
	110: "fiddle",     # #111 - Fiddle
	111: "shannai",    # #112 - Shanai
	112: "carillon",   # #113 - Tinkle Bell
	113: "agogo",      # #114 - Agogo
	114: "steeldrm",   # #115 - Steel Drums
	115: "woodblk",    # #116 - Woodblock
	116: "taiko",      # #117 - Taiko Drum
	117: "toms",       # #118 - Melodic Tom
	118: "syntom",     # #119 - Synth Drum
	119: "revcym",     # #120 - Reverse Cymbal
	120: "fx-fret",    # #121 - Guitar Fret Noise
	121: "fx-blow",    # #122 - Breath Noise
	122: "seashore",   # #123 - Seashore
	123: "jungle",     # #124 - Bird Tweet
	124: "telephon",   # #125 - Telephone Ring
	125: "helicptr",   # #126 - Helicopter
	126: "applause",   # #127 - Applause
	127: "pistol",     # #128 - Gunshot
	128: "blank",
	162: "kick1",      # #35 Acoustic Bass Drum
	163: "kick2",      # #36 Bass Drum 1
	164: "stickrim",   # #37 Side Stick
	165: "snare1",     # #38 Acoustic Snare
	166: "claps",      # #39 Hand Clap
	167: "snare2",     # #40 Electric Snare
	168: "tomlo2",     # #41 Low Floor Tom
	169: "hihatcl",    # #42 Closed Hi Hat
	170: "tomlo1",     # #43 High Floor Tom
	171: "hihatpd",    # #44 Pedal Hi-Hat
	172: "tommid2",    # #45 Low Tom
	173: "hihatop",    # #46 Open Hi-Hat
	174: "tommid1",    # #47 Low-Mid Tom
	175: "tomhi2",     # #48 Hi-Mid Tom
	176: "cymcrsh1",   # #49 Crash Cymbal 1
	177: "tomhi1",     # #50 High Tom
	178: "cymride1",   # #51 Ride Cymbal 1
	179: "cymchina",   # #52 Chinese Cymbal
	180: "cymbell",    # #53 Ride Bell
	181: "tamborin",   # #54 Tambourine
	182: "cymsplsh",   # #55 Splash Cymbal
	183: "cowbell",    # #56 Cowbell
	184: "cymcrsh2",   # #57 Crash Cymbal 2
	185: "vibslap",    # #58 Vibraslap
	186: "cymride2",   # #59 Ride Cymbal 2
	187: "bongohi",    # #60 Hi Bongo
	188: "bongolo",    # #61 Low Bongo
	189: "congahi1",   # #62 Mute Hi Conga
	190: "congahi2",   # #63 Open Hi Conga
	191: "congalo",    # #64 Low Conga
	192: "timbaleh",   # #65 High Timbale
	193: "timbalel",   # #66 Low Timbale
	194: "agogohi",    # #67 High Agogo
	195: "agogolo",    # #68 Low Agogo
	196: "cabasa",     # #69 Cabasa
	197: "maracas",    # #70 Maracas
	198: "whistle1",   # #71 Short Whistle
	199: "whistle2",   # #72 Long Whistle
	200: "guiro1",     # #73 Short Guiro
	201: "guiro2",     # #74 Long Guiro
	202: "clave",      # #75 Claves
	203: "woodblk1",   # #76 Hi Wood Block
	204: "woodblk2",   # #77 Low Wood Block
	205: "cuica1",     # #78 Mute Cuica
	206: "cuica2",     # #79 Open Cuica
	207: "triangl1",   # #80 Mute Triangle
	208: "triangl2",   # #81 Open Triangle
}

# These are the data sizes of the patch files distributed with the
# GUS drivers. These are used to calculate the size in RAM of the
# generated patch sets to check that they are within the limits.
# and check it is within the limit.

PATCH_FILE_SIZES = {
	"acbass": 5248,      "accordn": 9616,     "acguitar": 26080,
	"acpiano": 32256,    "agogo": 13696,      "agogohi": 3488,
	"agogolo": 3488,     "altosax": 5616,     "applause": 30064,
	"atmosphr": 31360,   "aurora": 31088,     "bagpipes": 7760,
	"banjo": 32016,      "barisax": 10544,    "basslead": 26496,
	"bassoon": 8000,     "belltree": 31888,   "blank": 1520,
	"bongohi": 3456,     "bongolo": 4448,     "bottle": 12368,
	"bowglass": 24688,   "britepno": 36000,   "cabasa": 8448,
	"calliope": 22992,   "carillon": 5888,    "castinet": 6016,
	"celeste": 9936,     "cello": 9120,       "charang": 45056,
	"chiflead": 31536,   "choir": 22480,      "church": 2144,
	"claps": 5696,       "clarinet": 9184,    "clave": 2352,
	"clavinet": 1440,    "cleangtr": 22768,   "concrtna": 8784,
	"congahi1": 4224,    "congahi2": 4704,    "congalo": 4704,
	"contraba": 4704,    "cowbell": 3168,     "crystal": 30224,
	"cuica1": 9344,      "cuica2": 12848,     "cymbell": 17248,
	"cymchina": 24112,   "cymcrsh1": 31520,   "cymcrsh2": 31040,
	"cymride1": 17664,   "cymride2": 17664,   "cymsplsh": 31520,
	"distgtr": 18848,    "doo": 8464,         "echovox": 14976,
	"englhorn": 12096,   "epiano1": 7344,     "epiano2": 21936,
	"fantasia": 23456,   "fiddle": 5904,      "flute": 6032,
	"fngrbass": 9744,    "frenchrn": 14128,   "freshair": 28992,
	"fretless": 2640,    "fx-blow": 28688,    "fx-fret": 13648,
	"ghostie": 31488,    "glocken": 5184,     "gtrharm": 4928,
	"guiro1": 4128,      "guiro2": 9248,      "halopad": 29984,
	"harmonca": 7408,    "harp": 11728,       "helicptr": 25008,
	"highq": 1808,       "hihatcl": 4560,     "hihatop": 20048,
	"hihatpd": 1808,     "hitbrass": 31520,   "homeorg": 992,
	"honky": 65680,      "hrpschrd": 3584,    "jazzgtr": 27712,
	"jingles": 16944,    "jungle": 13616,     "kalimba": 2208,
	"kick1": 4544,       "kick2": 5024,       "koto": 20832,
	"lead5th": 6464,     "maracas": 4560,     "marcato": 61232,
	"marimba": 2064,     "metalpad": 30288,   "metbell": 112,
	"metclick": 112,     "musicbox": 15312,   "mutegtr": 17008,
	"mutetrum": 9168,    "nyguitar": 19200,   "oboe": 3952,
	"ocarina": 1616,     "odguitar": 12640,   "orchhit": 14208,
	"percorg": 7520,     "piccolo": 4320,     "pickbass": 16416,
	"pistol": 18144,     "pizzcato": 19888,   "polysyn": 30224,
	"recorder": 2656,    "reedorg": 1568,     "revcym": 13536,
	"rockorg": 30288,    "santur": 21760,     "sawwave": 27056,
	"scratch1": 4384,    "scratch2": 2288,    "seashore": 31040,
	"shakazul": 31136,   "shaker": 3104,      "shamisen": 13136,
	"shannai": 9792,     "sitar": 18288,      "slap": 5856,
	"slapbas1": 27872,   "slapbas2": 20592,   "slowstr": 18192,
	"snare1": 8544,      "snare2": 4096,      "soundtrk": 19888,
	"sprnosax": 7072,    "sqrclick": 112,     "sqrwave": 15056,
	"startrak": 27376,   "steeldrm": 11952,   "stickrim": 2848,
	"sticks": 4224,      "surdo1": 9600,      "surdo2": 9600,
	"sweeper": 31216,    "synbass1": 6160,    "synbass2": 2928,
	"synbras1": 30704,   "synbras2": 30160,   "synpiano": 5456,
	"synstr1": 31216,    "synstr2": 16416,    "syntom": 30512,
	"taiko": 18672,      "tamborin": 8944,    "telephon": 4416,
	"tenorsax": 8448,    "timbaleh": 5264,    "timbalel": 9728,
	"timpani": 7072,     "tomhi1": 6576,      "tomhi2": 6560,
	"tomlo1": 6560,      "tomlo2": 9600,      "tommid1": 6560,
	"tommid2": 6560,     "toms": 6576,        "tremstr": 61232,
	"triangl1": 2224,    "triangl2": 15792,   "trombone": 12896,
	"trumpet": 6608,     "tuba": 5760,        "tubebell": 9120,
	"unicorn": 30096,    "vibes": 10640,      "vibslap": 9456,
	"viola": 27952,      "violin": 12160,     "voices": 14976,
	"voxlead": 14992,    "warmpad": 18080,    "whistle": 5872,
	"whistle1": 2000,    "whistle2": 928,     "woodblk": 3680,
	"woodblk1": 2352,    "woodblk2": 3680,    "woodflut": 1936,
	"xylophon": 9376,
}

# Groups of "similar sounding" instruments. The first instrument in each
# group is the "leader" and will be used as the fallback for other
# instruments in the group if they are not popular enough to be included.

# These groups are based on inaccurate stereotypes about musical instruments.
# If you're looking to improve the config, here's where to start.

SIMILAR_GROUPS = [
	# Pianos.
	('acpiano', 'britepno', 'synpiano', 'honky', 'epiano1', 'epiano2',
	 'hrpschrd', 'clavinet', 'celeste', 'glocken'),
	# Xylophone etc.
	('marimba', 'musicbox', 'vibes', 'xylophon', 'tubebell', 'santur',
	 'kalimba'),
	# Organs.
	('homeorg', 'percorg', 'rockorg', 'church', 'reedorg', 'accordn',
	 'harmonca', 'concrtna'),
	# Guitars.
	('nyguitar', 'acguitar', 'jazzgtr', 'cleangtr', 'mutegtr', 'odguitar',
	 'distgtr', 'gtrharm'),
	# Basses.
	('synbass2', 'acbass', 'fngrbass', 'pickbass', 'fretless', 'slapbas1',
	 'slapbas2', 'synbass1'),
	# Violin and similar string instruments.
	('violin', 'viola', 'cello', 'contraba', 'tremstr', 'pizzcato', 'harp',
	 'timpani'),
	# Other stringed (?)
	('marcato', 'slowstr', 'synstr1', 'synstr2', 'choir', 'doo', 'voices',
	 'orchhit'),
	# Trumpet and other brass.
	('trumpet', 'trombone', 'tuba', 'mutetrum', 'frenchrn', 'hitbrass',
	 'synbras1', 'synbras2'),
	# Reed instruments.
	('sprnosax', 'altosax', 'tenorsax', 'barisax', 'oboe', 'englhorn',
	 'bassoon', 'clarinet'),
	# Pipe instruments.
	('flute', 'piccolo', 'recorder', 'woodflut', 'bottle', 'shakazul',
	 'whistle', 'ocarina', 'bagpipes', 'fiddle', 'shannai'),
	# Leads.
	('sqrwave', 'sawwave', 'calliope', 'chiflead', 'charang', 'voxlead',
	 'lead5th', 'basslead', 'sitar', 'banjo', 'shamisen', 'koto'),
	# Special effects. Blank unless popular enough to appear.
	('blank','fantasia', 'warmpad', 'polysyn', 'ghostie', 'bowglass',
	 'metalpad', 'halopad', 'sweeper', 'aurora', 'soundtrk', 'crystal',
	 'atmosphr', 'freshair', 'unicorn', 'echovox', 'startrak', 'fx-fret',
	 'fx-blow', 'seashore', 'jungle', 'telephon', 'helicptr', 'applause',
	 'pistol'),

	# -- Percussion effects. This is probably where the current config is
	#    worst and in dire need of improvement.

	# Conga/kick
	('kick1', 'steeldrm', 'taiko', 'kick2', 'congahi1', 'congahi2',
	 'congalo', 'clave'),
	# Snare drums
	('snare1', 'stickrim', 'claps', 'snare2', 'tamborin', 'vibslap',
	 'bongohi', 'bongolo', 'timbaleh', 'timbalel', 'maracas', 'woodblk1',
	 'woodblk2'),
	# Hi-hat
	('hihatpd', 'carillon', 'agogo', 'woodblk', 'hihatcl', 'hihatop',
	 'cowbell', 'agogohi', 'agogolo', 'cabasa', 'whistle1', 'whistle2',
	 'guiro1', 'guiro2', 'cuica1', 'cuica2', 'triangl1', 'triangl2'),
	# Toms
	('tommid1', 'toms', 'syntom', 'tomlo2', 'tomlo1', 'tommid2', 'tomhi2',
	 'tomhi1'),
	# Cymbals.
	('cymcrsh1', 'revcym', 'cymride1', 'cymchina', 'cymbell', 'cymsplsh',
	 'cymcrsh2', 'cymride2'),
]




#!/usr/bin/python
from doom import Patch, Texture

import sys,re

if len(sys.argv) != 2:
	sys.stderr.write("usage: sw1_sw2.py <infile>\n")
	sys.exit()

infile = sys.argv[1]

# TODO: a generalized form of this para should probably be moved into the
# Texture class
texture1 = file(infile, "r").read()
textures = {}
current = None
for line in texture1.split("\n"):
	if len(line) == 0 or line[0] == ";" or line[0] == "#":
		continue
	elif line[0] == "*" and current:
		junk,name,y,x= line.split()
		current.patches.append(Patch(name,int(x),int(y)))
	else:
		line = line.split()
		current = Texture(line[0],line[1],line[2])
		textures[line[0]] = current

oldtex = file("../../textures/combined.txt", "r").read()
patches = []
texline = ''
for line in oldtex.split("\n"):
	# start of a patch line
	if len(line) > 0 and line[0] == "*":
		patches.append(line)
	else:
		# end of a texture definition and we had just one patch
		if 1 == len(patches):
			print patches[0]
			patches = []
			texline = ''
		# end of a texture definition,look up the patches
		elif texline != '':
			bits = texline.split()
			if bits[0] in textures:
				texture = textures[bits[0]]
				print '\n'.join(map(str,texture.patches))
				texline = ''
			else:
				sys.stderr.write("uh oh, no idea of patches for %s"%bits[0])

		# start of a texture definition
		if re.match(r'^[A-Z]', line):
			texline = line
		print line



#!/usr/bin/python
import os,sys,re

# this sucks

patches = [ x for x in os.listdir('.') if re.match(r'.*\.gif$', x) ]
patches.sort()

print '''<style type="text/css">
	div {
		float: left;
		width: 20%;
	}
</style>
'''
print ''.join(['<div><img src="%s" /><br />%s</div>' % (x,x) for x in patches])



#!/usr/bin/python

import sys
import gtk
import gtk.glade

from doom import Patch, Texture

class HellowWorldGTK:
	"""This is an Hello World GTK application"""
	
	def parse_texture_file(self,fname):
		texture1 = file(fname, "r").read()
		self.textures = {}
		self.wip_textures = {}
		textures = self.textures
		current = None
		for line in texture1.split("\n"):
			if len(line) == 0 or line[0] == ";" or line[0] == "#":
				continue
			elif line[0] == "*" and current:
				junk,name,y,x= line.split()
				current.patches.append(Patch(name,int(x),int(y)))
			else:
				line = line.split()
				current = Texture(line[0],line[1],line[2])
				self.wip_textures[line[0]] = Texture(line[0],line[1],line[2])
				textures[line[0]] = current
		# now weed out the 1-patch ones
		keys = self.textures.keys()
		for key in keys:
			if 1 == len(self.textures[key].patches):
				del self.textures[key]
				del self.wip_textures[key]

	def lhs_callback(self, treeview):
		offs,col = treeview.get_cursor()
		# TODO: sanity check the insane subscripting here
		row = treeview.get_model()[offs[0]][0]
		self.set_texture(row)
		# clear out the patch-list for this texture
		selected_patches = self.wTree.get_widget("selected_patches")
		model = selected_patches.get_model()
		model.clear()
		wip_texture = self.wip_textures[row]
		for patch in wip_texture.patches:
			model.append(None, [ patch.name, patch.xoff, patch.yoff ])
			

	def rhs_drag_data_get(self,widget,ctx,selection,targettype,eventtime):
		print "ZOMG"

	# patch selected on rhs
	def rhs_select_cb(self, rhs, path, view_column):
		model = rhs.get_model()
		row = model[path[0]][0]
		selected_patches = self.wTree.get_widget("selected_patches")
		model = selected_patches.get_model()
		model.append(None, [ row, 0, 0 ])
		# redraw the rhs. a bit hacky
		current_texture_name = self.current_texture()
		current_texture = self.wip_textures[current_texture_name]
		current_texture.patches.append(Patch(row,0,0))
		self.init_texture_pixbuf(self.wip_textures[current_texture_name])
		# this redraws the lhs too, unnecessarily
		self.set_texture(current_texture_name)

	def current_texture(self):
		"""return the name of the currently selected texture"""
		lhs = self.wTree.get_widget("texture_list")
		offs,col = lhs.get_cursor()
		lhs_model = lhs.get_model()
		return lhs_model[offs[0]][0]

	def rhs_cursor_cb(self, rhs):
		offs,col = rhs.get_cursor()
		rhs_model = rhs.get_model()
		row = rhs_model[offs[0]][0]
		
		image = self.wTree.get_widget("current_patch")
		image.set_from_file("../../patches/" + row.lower() + ".gif")
		# scale the texture up
		pixbuf = image.get_pixbuf()
		if pixbuf:
			scale = 2
			image.set_from_pixbuf(pixbuf.scale_simple(
				pixbuf.get_width()  * scale,
				pixbuf.get_height() * scale,
				gtk.gdk.INTERP_NEAREST
			))

	def init_texture_pixbuf(self, texture):
		for patch in texture.patches:
			# read the patches into pixbufs
			# a horrid hack to get them client->server side
			image = gtk.Image()
			if not self.patch_pixbufs.has_key(patch.name):
				image.set_from_file("../../patches/" + patch.name.lower() + ".gif")
				pb = image.get_pixbuf()
				self.patch_pixbufs[patch.name] = pb.add_alpha(True,chr(0),chr(255),chr(255))

		texbuf = gtk.gdk.Pixbuf(
			self.patch_pixbufs.values()[0].get_colorspace(),
			self.patch_pixbufs.values()[0].get_has_alpha(),
			self.patch_pixbufs.values()[0].get_bits_per_sample(),
			int(texture.width),
			int(texture.height))

		texbuf.fill(0x00ffffff)
		texbuf = texbuf.add_alpha(True,chr(0),chr(255),chr(255))
		texture.pixbuf = texbuf

		# copy each patch into the texture pixbuf
		for patch in texture.patches:

			dest_x = max(0, patch.xoff)
			dest_y = max(0, patch.yoff)
			dest_height = min(
				self.patch_pixbufs[patch.name].get_height(),
				texbuf.get_height() - dest_y
			)
			dest_width  = min(
				self.patch_pixbufs[patch.name].get_width(),
				texbuf.get_width()  - dest_x
			)
			if patch.xoff < 0:
				dest_width += patch.xoff
			if patch.yoff < 0:
				dest_height += patch.yoff
			offset_x = patch.xoff
			offset_y = patch.yoff
			pb = self.patch_pixbufs[patch.name]
			pb.composite(
				texbuf,
				dest_x, dest_y, dest_width, dest_height,
                offset_x, offset_y, 1, 1, # scale
				gtk.gdk.INTERP_NEAREST, 255)

	def set_texture(self, name):
		# parse the example texture, fetch the width,height;
		# create Patch objects and stuff them into a list
		texture = self.textures[name]
		image = self.wTree.get_widget("orig_texture")

		if not texture.pixbuf:
			self.init_texture_pixbuf(texture)

		image.set_from_pixbuf(texture.pixbuf)
		self.scale_up_texture(image)

		# now for the wip side
		wip_image = self.wTree.get_widget("wip_texture")
		wip_texture = self.wip_textures[name]
		if not wip_texture.pixbuf:
			self.init_texture_pixbuf(wip_texture)
		wip_image.set_from_pixbuf(wip_texture.pixbuf)
		self.scale_up_texture(wip_image)


	def scale_up_texture(self, image):
		# scale the texture up
		pixbuf = image.get_pixbuf()
		if pixbuf:
			scale = 2
			image.set_from_pixbuf(pixbuf.scale_simple(
				pixbuf.get_width()  * scale,
				pixbuf.get_height() * scale,
				gtk.gdk.INTERP_NEAREST
			))
	
	def patch_list_key(self,treeview,event):
		"""handle key press events on the patch list"""
		if gtk.keysyms.Delete == event.keyval:
			offs,col = treeview.get_cursor()
			# remove from data structure
			texture = self.wip_textures[self.current_texture()]
			del texture.patches[offs[0]]
			# remove from UI
			model = treeview.get_model()
			iter = model.get_iter(offs)
			model.remove(iter)
			# redraw RHS
			self.init_texture_pixbuf(texture)
			wip_image = self.wTree.get_widget("wip_texture")
			wip_image.set_from_pixbuf(texture.pixbuf)
			self.scale_up_texture(wip_image)

	def cell_callback(self, cellrenderertext, path, new_text):
		"""cell edited in patch list"""
		current_texture = self.wip_textures[self.current_texture()]
		path = int(path)
		new_text = int(new_text)
		if self.cellrendererwidth == cellrenderertext:
			current_texture.patches[path].xoff = new_text
			column = 1
		else:
			current_texture.patches[path].yoff = new_text
			column = 2
		self.init_texture_pixbuf(current_texture)
		wip_image = self.wTree.get_widget("wip_texture")
		wip_image.set_from_pixbuf(current_texture.pixbuf)
		self.scale_up_texture(wip_image)

		# update the model now
		selected_patches = self.wTree.get_widget("selected_patches")
		model = selected_patches.get_model()
		iter = model.get_iter(path)
		model.set_value(iter, column, new_text)


	def patch_list_reordered(self,treemodel,path,iter,new_order):
		print "ZOMG"


	def __init__(self):
		self.gladefile = "cleanroom.glade"
		self.wTree = gtk.glade.XML(self.gladefile,"window1")
		self.image1 = self.wTree.get_widget("orig_texture")
		
		window = self.wTree.get_widget("window1")
		window.resize(1024,768)

		self.patch_pixbufs = {}
		
		# read in the IWAD texture1 lump and populate our LHS list
		self.parse_texture_file("../../textures/combined.txt")
		lhs = self.wTree.get_widget("texture_list")
		treestore = gtk.TreeStore(str)
		tmp_texnames = self.textures.keys()
		tmp_texnames.sort()
		for name in tmp_texnames:
			if len(self.textures[name].patches) > 1:
				treestore.append(None, [ name ])
		column = gtk.TreeViewColumn('Texture name ')
		lhs.set_model(treestore)
		lhs.append_column(column)
		cell = gtk.CellRendererText()
		column.pack_start(cell, False)
		column.add_attribute(cell, "text", 0)
		lhs.connect("cursor-changed", self.lhs_callback)

		# prepare the patch list
		patch_list = self.wTree.get_widget("selected_patches")
		treestore = gtk.TreeStore(str,int,int)
		column = gtk.TreeViewColumn('Patch name')
		patch_list.set_model(treestore)
		patch_list.append_column(column)
		cell = gtk.CellRendererText()
		column.pack_start(cell, False)
		column.add_attribute(cell, "text", 0)

		cell = gtk.CellRendererText()
		self.cellrendererwidth = cell
		cell.set_property("editable", True)
		cell.connect("edited", self.cell_callback)
		column = gtk.TreeViewColumn('X offset')
		patch_list.append_column(column)
		column.pack_start(cell, False)
		column.add_attribute(cell, "text", 1)

		cell = gtk.CellRendererText()
		cell.set_property("editable", True)
		cell.connect("edited", self.cell_callback)
		column = gtk.TreeViewColumn('Y offset')
		patch_list.append_column(column)
		column.pack_start(cell, False)
		column.add_attribute(cell, "text", 2)

		patch_list.connect("key-press-event", self.patch_list_key)
		patch_list.set_reorderable(True)
		treestore.connect("rows-reordered", self.patch_list_reordered)

		# populate the RHS list with patch names
		# yes, I learnt perl once.
		patches = {}
		for texture in self.textures.values():
			for patch in texture.patches:
				patches[patch.name] = 1
		patches = patches.keys()
		rhs = self.wTree.get_widget("all_patch_tree")
		treestore = gtk.TreeStore(str)
		patches.sort()
		for name in patches:
			treestore.append(None, [ name ])
		column = gtk.TreeViewColumn('Patch name')
		rhs.set_model(treestore)
		rhs.append_column(column)
		cell = gtk.CellRendererText()
		column.pack_start(cell, False)
		column.add_attribute(cell, "text", 0)
		rhs.connect("cursor-changed", self.rhs_cursor_cb)
		rhs.connect("row-activated", self.rhs_select_cb)

		image = self.wTree.get_widget("current_patch")
		image.drag_source_set(gtk.gdk.BUTTON1_MASK, [('text/plain', 0,
			80)], 0) # 80 = target type text
		image.connect("drag_data_get", self.rhs_drag_data_get)

		self.set_up_progressbar()

		self.wTree.get_widget("window1").connect("destroy", gtk.main_quit)
		self.wTree.get_widget("quit_menu_item").connect("activate", gtk.main_quit)
		self.wTree.get_widget("saveas_menu_item").connect("activate", self.saveas_activated)
		self.wTree.get_widget("save_menu_item").connect("activate", self.save_activated)
		self.wTree.get_widget("open_menu_item").connect("activate", self.open_activated)

		# select the top-most texture
		lhs.set_cursor( (0,) , None, False)

	def set_up_progressbar(self):
		"""We've "done" any textures in wip set with >0 patches"""
		bar = self.wTree.get_widget("progressbar1")
		done = len(filter(lambda x: len(x.patches) > 0, self.wip_textures.values()))
		bar.set_fraction(float(done) / len(self.textures))
		bar.set_text("%d/%d" % (done, len(self.textures)))

	def saveas_activated(self, arg):
		filesel = gtk.FileChooserDialog(action=gtk.FILE_CHOOSER_ACTION_SAVE,
			buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
				gtk.STOCK_SAVE, gtk.RESPONSE_OK))
		if filesel.run() == gtk.RESPONSE_OK:
			self.filename = filesel.get_filename()
		filesel.destroy()
		self.save_activated(arg)

	def save_activated(self, arg):
		if not hasattr(self, "filename"):
			self.saveas_activated(arg)
		saveme = self.wip_textures.values()
		saveme.sort(lambda a,b: cmp(a.name,b.name))
		writetome = open(self.filename,"w")
		writetome.write("".join(map(str,saveme)))
		writetome.close()

	def open_activated(self,arg):
		filesel = \
		gtk.FileChooserDialog(action=gtk.FILE_CHOOSER_ACTION_OPEN,
			buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
				gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		if filesel.run() == gtk.RESPONSE_OK:
			self.filename = filesel.get_filename()
		filesel.destroy()
		print self.filename
		
		texture1 = file(self.filename, "r").read()
		self.wip_textures = {}
		current = None
		for line in texture1.split("\n"):
			if len(line) == 0 or line[0] == ";" or line[0] == "#":
				continue
			elif line[0] == "*" and current:
				junk,name,y,x= line.split()
				current.patches.append(Patch(name,int(x),int(y)))
			else:
				line = line.split()
				current = Texture(line[0],line[1],line[2])
				self.wip_textures[line[0]] = current
		self.set_up_progressbar()

if __name__ == "__main__":
	hwg = HellowWorldGTK()
	gtk.main()



#!/usr/bin/env python
# Script to generate the PLAYPAL lump used by the Doom engine, specifically the
# which contains 14 alternative palettes which are used for various
# environmental effects. The base palette from which these are derived is either
# generated, or taken from a file.
#
# This is a Python version of the original Perl script.
#
# Copyright (C) 2008  Simon Howard
# Copyright (C) 2001  Colin Phipps <cphipps@doomworld.com>
# Parts copyright (C) 1999 by id Software (http://www.idsoftware.com/)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import sys
import struct

# IHS (Intensity Hue Saturation) to RGB conversion, utility function
#
# Obtained from a web page, which credited the following for the algorithm
#  Kruse, F.A. and G.L. Raines, 1984. "A Technique For Enhancing Digital
#  Colour Images by Contrast Stretching in Munsell Colour Space",
#  Proceedings of the International Symposium on Remote Sensing of
#  Environment, 3rd Thematic Conference, Environmental Research Institute
#  of Michigan, Colorado Springs, Colorado, pp. 755-773.
#  Bonham-Carter, Graeme F., 1994.  Geographic Informations Systems for
#  Geoscientists: Modelling with GIS. Computer Methods in the
#  Geosciences, Volume 13, published by Pergamon (Elsevier Science Ltd),
#  pp. 120-125.

R2 = 1.0 / 2
R3 = 1.0 / 3
R6 = 1.0 / 6
PI = 3.141592

def ihs_to_rgb(i, h, s):
	i = (i * 422) / 255
	h = (h * 2 * PI) / 255
	s = (s * 208.2066) / 255

	b, x = s * math.cos(h), s * math.sin(h)

	return (R3 * i - R6 * b - R2 * x,
		R3 * i - R6 * b + R2 * x,
		R3 * i + R6 * 2 * b)

# New palette builder

def make_pal_range(i, h, s, n):

	map_function = lambda x: ihs_to_rgb(i * (n - x) / n,
	                                    h,
	                                    s * (n - x) / n),

	return map(map_function, range(n))

# Very crude traversal of the IHS colour ball

def make_palette_new():
	result = []
	
	result += make_pal_range(255, 0, 0, 32)

	for i in range(7):
		result += make_pal_range(127, 171 + (i + 1) * 256 / 7, 255, 16)
	
	for i in range(7):
		result += make_pal_range(256, (i + 1) * 256 / 7, 127, 16)

# Return palette read from named file

def read_palette(filename):
	f = open(filename, "rb")

	colors = []

	for i in range(256):
		data = f.read(3)
		color = struct.unpack("BBB", data)

		colors.append(color)

	f.close()

	return colors

def make_palette(filename):
	if filename is None:
		return make_palette_new
	else:
		return read_palette(filename)

# Old palette builder
#sub make_pal_range($$$$$$)
#{
#  my ($rs,$gs,$bs,$re,$ge,$be) = @_;
#  return map { my $e = $_/16; my $s = 1-$e;
#  [$rs*$s + $re*$e, $gs*$s + $ge*$e, $bs*$s + $be * $e] } (1..16);
#}
#
#sub make_palette ()
#{
#  my @p = (
#  make_pal_range(0,0,0,0,0,0), # hmmm
#  make_pal_range(255,255,255,255,0,0), # pinks
#  make_pal_range(255,0,0,0,0,0), # dull reds
#  make_pal_range(255,128,255,192,192,0), # yellows
#  make_pal_range(255,255,0,0,0,0), # yellows
#  make_pal_range(255,255,255,0,0,0), # white
#  make_pal_range(127,127,127,0,0,0), # gray
#  make_pal_range(255,255,255,0,255,0), # light greens
#  make_pal_range(0,255,0,0,0,0), # greens
#  make_pal_range(0,0,0,0,0,0), # hmmm
#  make_pal_range(0,0,255,0,0,0), # dark blues
#  make_pal_range(255,255,255,0,0,255), # bright blues
#  make_pal_range(255,0,255,0,0,0), # magenta
#  make_pal_range(0,255,255,0,0,0), # cyan
#  make_pal_range(0,0,0,0,0,0), # hmmm
#  make_pal_range(0,0,0,0,0,0)); # hmmm
#  return \@p;
#}

# Now the PLAYPAL stuff - take the main palette and construct biased versions
# for the palette translation stuff

# Bias an entire palette

def bias_palette_towards(palette, target, p):

	def bias_rgb(rgb):
		r = []

		for i in range(3):
			r.append(rgb[i] * (1 - p) + target[i] * p)

		return r

	return map(bias_rgb, palette)

# Encode palette in the 3-byte RGB triples format expected by the engine

def clamp_pixval(v):
	if v < 0:
		return 0
	elif v > 255:
		return 255
	else:
		return int(v)

def output_palette(pal):

	for color in palette:
		color = tuple(map(clamp_pixval, color))
		
		encoded = struct.pack("BBB", *color)
		os.write(sys.stdout.fileno(), encoded)

# Main program - make a base palette, then do the biased versions

if len(sys.argv) < 2:
	print("Usage: %s <base filename> > playpal.lmp" % sys.argv[0])
	sys.exit(1)

base_pal = read_palette(sys.argv[1])

# From st_stuff.c, Copyright 1999 id Software, license GPL
#define STARTREDPALS         1
#define STARTBONUSPALS       9
#define NUMREDPALS           8
#define NUMBONUSPALS         4
#define RADIATIONPAL         13

palettes = []

# Normal palette

palettes.append(base_pal)

# STARTREDPALS

for i in range(8):
	p = (i + 1) / 8.0

	palettes.append(bias_palette_towards(base_pal, (255, 0, 0), p))

# STARTBONUSPALS

for i in range(4):
	p = (i + 1) * 0.4 / 4

	palettes.append(bias_palette_towards(base_pal, (128, 128, 128), p))

# RADIATIONPAL

palettes.append(bias_palette_towards(base_pal, (0, 255, 0), 0.2))

for palette in palettes:
	output_palette(palette)




#!/usr/bin/env python
#
# Copyright (c) 2011, 2012
# Contributors to the Freedoom project.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the freedoom project nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ----------------------------------------------------------------------
#
# Module to load Creative SBI OPL instrument files.
#

import struct
import sys

HEADER_VALUE = "SBI\x1a"

FIELDS = [
	"m_am_vibrato_eg",
	"c_am_vibrato_eg",
	"m_ksl_volume",
	"c_ksl_volume",
	"m_attack_decay",
	"c_attack_decay",
	"m_sustain_release",
	"c_sustain_release",
	"m_waveform",
	"c_waveform",
	"feedback_fm"
]

def read(filename):
	f = open(filename)
	data = f.read()
	f.close()

	header, name = struct.unpack("4s32s", data[0:36])

	if header != HEADER_VALUE:
		raise Exception("Invalid header for SBI file!")

	instr_data = data[36:]
	result = { "name": name.rstrip("\0") }

	for i in range(len(FIELDS)):
		result[FIELDS[i]], = struct.unpack("B", instr_data[i:i+1])

	return result

def write(filename, data):
	f = open(filename, 'w')

	f.write(struct.pack("4s32s", HEADER_VALUE, data["name"]))

	for field in FIELDS:
		f.write(struct.pack("B", data[field]))
	for x in range(16 - len(FIELDS)):
		f.write(struct.pack("B", 0))

	f.close()

if __name__ == "__main__":
	for filename in sys.argv[1:]:
		print filename
		print read(filename)




#!/usr/bin/env python
#
# Copyright (c) 2011, 2012
# Contributors to the Freedoom project.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the freedoom project nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ----------------------------------------------------------------------
#
# Config file for GENMIDI file generation.
#
# This specifies the instrument files that should be used to generate
# the GENMIDI lump. Instrument files can be in SBI or A2I format.
# Instrument files must not use OPL3 features and it is an error to do so.
#
# A simple instrument specification looks like this:
#
#   Instrument("filename.sbi")
#
# For a double-voice instrument (two instrument files played at the same
# time), do this:
#
#   Instrument("file1.sbi", "file2.sbi")
#
# To tune the instruments, it's possible to apply an offset to the notes
# that are played. For example, to force all notes down by one octave:
#
#   Instrument("file1.sbi", off1=-12)
#
# This can be controlled individually for double-voice instruments, eg.
#
#   Instrument("file1.sbi", "file2.sbi", off1=-12, off2=+6)
#
# When an instrument is played, the note usually comes from the MIDI
# event. Some instruments (especially percussion) always play the same
# note. To specify a fixed note, do this:
#
#   Instrument("filename.sbi", note=On5.B)
#
# Value is a MIDI note number; see midi.py for constant definitions.

from instrument import Instrument, NullInstrument
from midi import *

# General MIDI instruments:

INSTRUMENTS = [
	Instrument("instr001.sbi"),             # #001 - Acoustic Grand Piano
	Instrument("instr002.sbi"),             # #002 - Bright Acoustic Piano
	Instrument("instr003.sbi"),             # #003 - Electric Grand Piano
	Instrument("instr004.sbi",              # #004 - Honky-tonk Piano
	           "instr004-2.sbi"),
	Instrument("instr005.sbi",              # #005 - Electric Piano 1
	           "instr005-2.sbi"),
	Instrument("instr006.sbi", off1=-12),   # #006 - Electric Piano 2
	Instrument("instr007.sbi"),             # #007 - Harpsichord
	Instrument("instr008.sbi", off1=-12),   # #008 - Clavi
	Instrument("instr009.sbi", off1=-12),   # #009 - Celesta
	Instrument("instr010.sbi", off1=-12),   # #010 - Glockenspiel
	Instrument("instr011.sbi", off1=-12),   # #011 - Music Box
	Instrument("instr012.sbi", off1=-12),   # #012 - Vibraphone
	Instrument("instr013.sbi", off1=-12),   # #013 - Marimba
	Instrument("instr014.sbi", off1=-12),   # #014 - Xylophone
	Instrument("instr015.sbi", off1=-12),   # #015 - Tubular Bells
	Instrument("instr016.sbi", off1=-12),   # #016 - Dulcimer
	Instrument("instr017.sbi", off1=-12),   # #017 - Drawbar Organ
	Instrument("instr018.sbi", off1=-12),   # #018 - Percussive Organ
	Instrument("instr019.sbi", off1=-12),   # #019 - Rock Organ
	Instrument("instr020.sbi", off1=-12),   # #020 - Church Organ
	Instrument("instr021.sbi", off1=-12),   # #021 - Reed Organ
	Instrument("instr022.sbi", off1=-12),   # #022 - Accordion
	Instrument("instr023.sbi", off1=-12),   # #023 - Harmonica
	Instrument("instr024.sbi", off1=-12),   # #024 - Tango Accordion
	Instrument("instr025.sbi", off1=-12),   # #025 - Acoustic Guitar (nylon)
	Instrument("instr026.sbi", off1=-12),   # #026 - Acoustic Guitar (steel)
	Instrument("instr027.sbi", off1=-12),   # #027 - Electric Guitar (jazz)
	Instrument("instr028.sbi", off1=-12),   # #028 - Electric Guitar (clean)
	Instrument("instr029.sbi", off1=-12),   # #029 - Electric Guitar (muted)
	Instrument("instr030.sbi", off1=-12),   # #030 - Overdriven Guitar
	Instrument("instr031.sbi", off1=-12),   # #031 - Distortion Guitar
	Instrument("instr032.sbi", off1=-12),   # #032 - Guitar harmonics
	Instrument("instr033.sbi", off1=-12),   # #033 - Acoustic Bass
	Instrument("instr034.sbi", off1=-12),   # #034 - Electric Bass (finger)
	Instrument("instr035.sbi", off1=-12),   # #035 - Electric Bass (pick)
	Instrument("instr036.sbi", off1=-12),   # #036 - Fretless Bass
	Instrument("instr037.sbi", off1=-12),   # #037 - Slap Bass 1
	Instrument("instr038.sbi", off1=-12),   # #038 - Slap Bass 2
	Instrument("instr039.sbi", off1=-12),   # #039 - Synth Bass 1
	Instrument("instr040.sbi", off1=-12),   # #040 - Synth Bass 2
	Instrument("instr041.sbi", off1=-12),   # #041 - Violin
	Instrument("instr042.sbi", off1=-12),   # #042 - Viola
	Instrument("instr043.sbi", off1=-12),   # #043 - Cello
	Instrument("instr044.sbi", off1=-12),   # #044 - Contrabass
	Instrument("instr045.sbi", off1=-12),   # #045 - Tremolo Strings
	Instrument("instr046.sbi", off1=-12),   # #046 - Pizzicato Strings
	Instrument("instr047.sbi", off1=-12),   # #047 - Orchestral Harp
	Instrument("instr048.sbi", off1=-12),   # #048 - Timpani
	Instrument("instr049.sbi", off1=-12),   # #049 - String Ensemble 1
	Instrument("instr050.sbi", off1=-12),   # #050 - String Ensemble 2
	Instrument("instr051.sbi", off1=-12),   # #051 - SynthStrings 1
	Instrument("instr052.sbi", off1=-12),   # #052 - SynthStrings 2
	Instrument("instr053.sbi", off1=-12),   # #053 - Choir Aahs
	Instrument("instr054.sbi", off1=-12),   # #054 - Voice Oohs
	Instrument("instr055.sbi", off1=-12),   # #055 - Synth Voice
	Instrument("instr056.sbi", off1=-12),   # #056 - Orchestra Hit
	Instrument("instr057.sbi", off1=-12),   # #057 - Trumpet
	Instrument("instr058.sbi", off1=-12),   # #058 - Trombone
	Instrument("instr059.sbi", off1=-12),   # #059 - Tuba
	Instrument("instr060.sbi", off1=-12),   # #060 - Muted Trumpet
	Instrument("instr061.sbi", off1=-12),   # #061 - French Horn
	Instrument("instr062.sbi", off1=-12),   # #062 - Brass Section
	Instrument("instr063.sbi", off1=-12),   # #063 - SynthBrass 1
	Instrument("instr064.sbi", off1=-12),   # #064 - SynthBrass 2
	Instrument("instr065.sbi", off1=-12),   # #065 - Soprano Sax
	Instrument("instr066.sbi", off1=-12),   # #066 - Alto Sax
	Instrument("instr067.sbi", off1=-12),   # #067 - Tenor Sax
	Instrument("instr068.sbi", off1=-12),   # #068 - Baritone Sax
	Instrument("instr069.sbi", off1=-12),   # #069 - Oboe
	Instrument("instr070.sbi", off1=-12),   # #070 - English Horn
	Instrument("instr071.sbi", off1=-12),   # #071 - Bassoon
	Instrument("instr072.sbi", off1=-12),   # #072 - Clarinet
	Instrument("instr073.sbi", off1=-12),   # #073 - Piccolo
	Instrument("instr074.sbi", off1=-12),   # #074 - Flute
	Instrument("instr075.sbi", off1=-12),   # #075 - Recorder
	Instrument("instr076.sbi", off1=-12),   # #076 - Pan Flute
	Instrument("instr077.sbi", off1=-12),   # #077 - Blown Bottle
	Instrument("instr078.sbi", off1=-12),   # #078 - Shakuhachi
	Instrument("instr079.sbi", off1=-12),   # #079 - Whistle
	Instrument("instr080.sbi", off1=-12),   # #080 - Ocarina
	Instrument("instr081.sbi", off1=-12),   # #081 - Lead 1 (square)
	Instrument("instr082.sbi"),             # #082 - Lead 2 (sawtooth)
	Instrument("instr083.sbi", off1=-12),   # #083 - Lead 3 (calliope)
	Instrument("instr084.sbi", off1=-12),   # #084 - Lead 4 (chiff)
	Instrument("instr085.sbi", off1=-12),   # #085 - Lead 5 (charang)
	Instrument("instr086.sbi", off1=-12),   # #086 - Lead 6 (voice)
	Instrument("instr087.sbi", off1=-12),   # #087 - Lead 7 (fifths)
	Instrument("instr088.sbi", off1=-12),   # #088 - Lead 8 (bass + lead)
	Instrument("instr089.sbi", off1=-12),   # #089 - Pad 1 (new age)
	Instrument("instr090.sbi", off1=-12),   # #090 - Pad 2 (warm)
	Instrument("instr091.sbi", off1=-12),   # #091 - Pad 3 (polysynth)
	Instrument("instr092.sbi", off1=-12),   # #092 - Pad 4 (choir)
	Instrument("instr093.sbi", off1=-12),   # #093 - Pad 5 (bowed)
	Instrument("instr094.sbi", off1=-12),   # #094 - Pad 6 (metallic)
	Instrument("instr095.sbi", off1=-12),   # #095 - Pad 7 (halo)
	Instrument("instr096.sbi", off1=-12),   # #096 - Pad 8 (sweep)
	Instrument("instr097.sbi", off1=-12),   # #097 - FX 1 (rain)
	Instrument("instr098.sbi", off1=-12),   # #098 - FX 2 (soundtrack)
	Instrument("instr099.sbi", off1=-12),   # #099 - FX 3 (crystal)
	Instrument("instr100.sbi", off1=-12),   # #100 - FX 4 (atmosphere)
	Instrument("instr101.sbi", off1=-12),   # #101 - FX 5 (brightness)
	Instrument("instr102.sbi", off1=-12),   # #102 - FX 6 (goblins)
	Instrument("instr103.sbi", off1=-12),   # #103 - FX 7 (echoes)
	Instrument("instr104.sbi", off1=-12),   # #104 - FX 8 (sci-fi)
	Instrument("instr105.sbi", off1=-12),   # #105 - Sitar
	Instrument("instr106.sbi", off1=-12),   # #106 - Banjo
	Instrument("instr107.sbi", off1=-12),   # #107 - Shamisen
	Instrument("instr108.sbi", off1=-12),   # #108 - Koto
	Instrument("instr109.sbi", off1=-12),   # #109 - Kalimba
	Instrument("instr110.sbi", off1=-12),   # #110 - Bag pipe
	Instrument("instr111.sbi", off1=-12),   # #111 - Fiddle
	Instrument("instr112.sbi", off1=-12),   # #112 - Shanai
	Instrument("instr113.sbi", off1=-12),   # #113 - Tinkle Bell
	Instrument("instr114.sbi", off1=-12),   # #114 - Agogo
	Instrument("instr115.sbi", off1=-12),   # #115 - Steel Drums
	Instrument("instr116.sbi", off1=-12),   # #116 - Woodblock
	Instrument("instr117.sbi", off1=-12),   # #117 - Taiko Drum
	Instrument("instr118.sbi", off1=-12),   # #118 - Melodic Tom
	Instrument("instr119.sbi", off1=-12),   # #119 - Synth Drum
	Instrument("instr120.sbi", off1=-12),   # #120 - Reverse Cymbal
	Instrument("instr121.sbi", off1=-12),   # #121 - Guitar Fret Noise
	Instrument("instr122.sbi", off1=-12),   # #122 - Breath Noise
	Instrument("instr123.sbi", off1=-12),   # #123 - Seashore
	Instrument("instr124.sbi", off1=-12),   # #124 - Bird Tweet
	Instrument("instr125.sbi", off1=-12),   # #125 - Telephone Ring
	Instrument("instr126.sbi", off1=-12),   # #126 - Helicopter
	Instrument("instr127.sbi", off1=-12),   # #127 - Applause
	Instrument("instr128.sbi", off1=-12),   # #128 - Gunshot
]

# Percussion instruments:
# Note: Many of these instruments have not been defined yet.
# As these are percussion instruments, a note number should ALWAYS be
# specified.

PERCUSSION = [
	Instrument("perc35.sbi", note=On4.A),   # #35 Acoustic Bass Drum
	Instrument("perc36.sbi", note=On4.A),   # #36 Bass Drum 1
	Instrument("perc37.sbi", note=On1.C),   # #37 Side Stick
	Instrument("perc38.sbi", note=On3.Gs),  # #38 Acoustic Snare
	Instrument("perc39.sbi", note=O3.C),    # #39 Hand Clap
	Instrument("perc40.sbi", note=On1.Cs),  # #40 Electric Snare
	Instrument("perc41.sbi", note=On3.D),   # #41 Low Floor Tom
	Instrument("perc42.sbi", note=O1.Gs),   # #42 Closed Hi Hat
	Instrument("perc43.sbi", note=On3.Gs),  # #43 High Floor Tom
	Instrument("perc44.sbi", note=O1.Gs),   # #44 Pedal Hi-Hat
	Instrument("perc45.sbi", note=On2.C),   # #45 Low Tom
	Instrument("perc46.sbi", note=O1.Gs),   # #46 Open Hi-Hat
	Instrument("perc47.sbi", note=On2.Fs),  # #47 Low-Mid Tom
	Instrument("perc48.sbi", note=On2.A),   # #48 Hi-Mid Tom
	Instrument("perc49.sbi", note=On1.C),   # #49 Crash Cymbal 1
	Instrument("perc50.sbi", note=On1.Cs),  # #50 High Tom
	Instrument("perc51.sbi", note=On1.B),   # #51 Ride Cymbal 1
	Instrument("perc52.sbi", note=On1.C),   # #52 Chinese Cymbal
	Instrument("perc53.sbi", note=O1.E),    # #53 Ride Bell
	Instrument("perc54.sbi", note=O0.E),    # #54 Tambourine
	NullInstrument,                         # TODO - #55 Splash Cymbal
	NullInstrument,                         # TODO - #56 Cowbell
	Instrument("perc57.sbi", note=On1.As),  # #57 Crash Cymbal 2
	NullInstrument,                         # TODO - #58 Vibraslap
	Instrument("perc59.sbi", note=O0.E),    # #59 Ride Cymbal 2
	NullInstrument,                         # TODO - #60 Hi Bongo
	NullInstrument,                         # TODO - #61 Low Bongo
	NullInstrument,                         # TODO - #62 Mute Hi Conga
	NullInstrument,                         # TODO - #63 Open Hi Conga
	NullInstrument,                         # TODO - #64 Low Conga
	NullInstrument,                         # TODO - #65 High Timbale
	NullInstrument,                         # TODO - #66 Low Timbale
	NullInstrument,                         # TODO - #67 High Agogo
	NullInstrument,                         # TODO - #68 Low Agogo
	NullInstrument,                         # TODO - #69 Cabasa
	Instrument("perc70.sbi", note=On5.E),   # #70 Maracas
	Instrument("perc71.sbi", note=On5.E),   # #71 Short Whistle
	Instrument("perc72.sbi", note=On5.E),   # #72 Long Whistle
	Instrument("perc73.sbi", note=On5.E),   # #73 Short Guiro
	Instrument("perc74.sbi", note=On5.E),   # #74 Long Guiro
	Instrument("perc75.sbi", note=On5.E),   # #75 Claves
	Instrument("perc76.sbi", note=On5.E),   # #76 Hi Wood Block
	Instrument("perc77.sbi", note=On5.E),   # #77 Low Wood Block
	Instrument("perc78.sbi", note=On5.E),   # #78 Mute Cuica
	Instrument("perc79.sbi", note=On5.E),   # #79 Open Cuica
	Instrument("perc80.sbi", note=On5.E),   # #80 Mute Triangle
	Instrument("perc81.sbi", note=On5.E),   # #81 Open Triangle
]




#
# Copyright (c) 2013
# Contributors to the Freedoom project.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the freedoom project nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ----------------------------------------------------------------------
#
# Configuration file for textgen. This file defines the graphic lumps
# that are generated, and the text to show in each one.
#

import re

# Adjustments for character position based on character pairs. Some
# pairs of characters can fit more snugly together, which looks more
# visually appealing. This is highly dependent on the font graphics,
# and if the font is changed this probably needs to be redone.

FONT_KERNING_RULES = {
	# Right character fits under left character:
	r'p[aj\.]': -3,
	r'P[a\.]': -4,
	r'[PVW][AJj\.]': -4,
	r't[ajJ\.]': -4,
	r'f[aj\.]': -2,

	# Some capital letters have overhangs that the 'lower case'
	# characters can fit under:
	r'C[Ja-z\.]': -2,
	r'F[Ja-z\.]': -3,
	r'T[Ja-z\.]': -5,
	r'W[Ja-z\.]': -2,
	r'S[Ja-z\.]': -1,
	r'V[a-z\.]': -3,

	# Left character fits under right character:
	r'[alAL][ty479]': -3,
	r'a[vwVW]': -2,
	r'A[VW]': -3,
	r'A[vw]': -2,
	r'r[ty479]': -2,
	r'[vwVW][Aa]': -2,
	r'[Yypv][jJ]': -2,

	# Extra space needed:
	r'[OUu][Pp]': +1,
}

white_graphics = {
	'wibp1': 'P1',
	'wibp2': 'P2',
	'wibp3': 'P3',
	'wibp4': 'P4',
	'wicolon': ':',
	'wiminus': '-',

	# Note: level names are also included in this dictionary, with
	# the data added programatically from the DEHACKED lump, see
	# code below.
}

blue_graphics = {
	'm_disopt': 'DISPLAY OPTIONS',
	'm_episod': 'Choose Episode:',
	'm_optttl': 'OPTIONS',
	'm_skill': 'Choose Skill Level:',
}

red_graphics = {
	'm_ngame': 'New Game',
	'm_option': 'Options',
	'm_loadg': 'Load Game',
	'm_saveg': 'Save Game',
	'm_rdthis': 'Read This!',
	'm_quitg': 'Quit Game',

	'm_newg': 'NEW GAME',
	'm_epi1': 'First Episode',
	'm_epi2': 'Second Episode',
	'm_epi3': 'Third Episode',
	'm_epi4': 'Double Impact',

	'm_jkill': 'Please don\'t kill me!',
	'm_rough': 'Will this hurt?',
	'm_hurt': 'Bring on the pain.',
	'm_ultra': 'Extreme Carnage',
	'm_nmare': 'INSANITY!',

	'm_lgttl': 'LOAD GAME',
	'm_sgttl': 'SAVE GAME',

	'm_endgam': 'End Game',
	'm_messg': 'Messages:',
	'm_msgoff': 'off',
	'm_msgon': 'on',
	'm_msens': 'Mouse Sensitivity',
	'm_detail': 'Graphic Detail:',
	'm_gdhigh': 'high',
	'm_gdlow': 'low',
	'm_scrnsz': 'Screen Size',

	'm_svol': 'Sound Volume',
	'm_sfxvol': 'Sfx Volume',
	'm_musvol': 'Music Volume',

	'm_disp': 'Display',

	'wif': 'finished',
	'wiostk': 'kills',
	'wiosti': 'items',
	'wiscrt2': 'secret',
	'wiosts': 'scrt',
	'wifrgs': 'frgs',

	'witime': 'Time:',
	'wisucks': 'sucks',
	'wimstt': 'Total:',
	'wipar': 'Par:',
	'wip1': 'P1', 'wip2': 'P2', 'wip3': 'P3', 'wip4': 'P4',
	'wiostf': 'f.',
	'wimstar': 'you',
	'winum0': '0', 'winum1': '1', 'winum2': '2', 'winum3': '3',
	'winum4': '4', 'winum5': '5', 'winum6': '6', 'winum7': '7',
	'winum8': '8', 'winum9': '9',
	'wipcnt': '%',
	'wienter': 'ENTERING',

	'm_pause': 'pause',

	# Extra graphics used in PrBoom's menus. Generate these as well
	# so that when we play in PrBoom the menus look consistent.
	'prboom': 'PrBoom',
	'm_generl': 'General',
	'm_setup': 'Setup',
	'm_keybnd': 'Key Bindings',
	'm_weap': 'Weapons',
	'm_stat': 'Status Bar/HUD',
	'm_auto': 'Automap',
	'm_enem': 'Enemies',
	'm_mess': 'Messages',
	'm_chat': 'Chat Strings',

	'm_horsen': 'horizontal',
	'm_versen': 'vertical',
	'm_loksen': 'mouse look',
	'm_accel': 'acceleration',

	# Extra graphics from SMMU/Eternity Engine:
	'm_about': 'about',
	'm_chatm': 'Chat Strings',
	'm_compat': 'Compatibility',
	'm_demos': 'demos',
	'm_dmflag': 'deathmatch flags',
	'm_etcopt': 'eternity options',
	'm_feat': 'Features',
	'm_gset': 'game settings',
	'm_hud': 'heads up display',
	'm_joyset': 'joysticks',
	'm_ldsv': 'Load/Save',
	'm_menus': 'Menu Options',
	'm_mouse': 'mouse options',
	'm_multi': 'multiplayer',
	'm_player': 'player setup',
	'm_serial': 'serial connection',
	'm_sound': 'sound options',
	'm_status': 'status bar',
	'm_tcpip': 'tcp/ip connection',
	'm_video': 'video options',
	'm_wad': 'load wad',
	'm_wadopt': 'wad options',
}

def read_bex_lump(filename):
	"""Read the BEX (Dehacked) lump from the given filename.

	Returns:
	    Dictionary mapping from name to value.
	"""
	result = {}
	with open(filename) as f:
		for line in f:
			# Ignore comments:
			line = line.strip()
			if len(line) == 0 or line[0] in '#;':
				continue
			# Just split on '=' and interpret that as an
			# assignment. This is primitive and doesn't read
			# like a full BEX parser should, but it's good
			# enough for our purposes here.
			assign = line.split('=', 2)
			if len(assign) != 2:
				continue
			result[assign[0].strip()] = assign[1].strip()
	return result

def update_from_bex(config, bexdata):
	"""Update the given config dictionary with data from a BEX lump.

	Args:
	    config: Dictionary of lumps, mapping from lump name to
	        text string to contain in it.
	    bexdata: Dictionary of data read from BEX lump.
	"""
	def update_lump(lumpname, bexname):
		if bexname not in bexdata:
			raise Exception('Level name %s not defined in '
			                'DEHACKED lump!' % bexname)
		# Strip "MAP01: " or "E1M2: " etc. from start, if present:
		levelname = re.sub('^\w*\d:\s*', '', bexdata[bexname])
		config[lumpname] = levelname

	for e in range(4):
		for m in range(9):
			# HUSTR_E1M1 from BEX => wilv00
			update_lump('wilv%i%i' % (e, m),
			            'HUSTR_E%iM%i' % (e + 1, m + 1))

	for m in range(32):
		# HUSTR_1 => cwilv00
		update_lump('cwilv%02i' % m,
		            'HUSTR_%i' % (m + 1))

update_from_bex(white_graphics, read_bex_lump('../../lumps/dehacked.lmp'))




#!/usr/bin/python

# cleanroom.py: a clean-room IWAD texture1 lump constructor
from doom import Patch, Texture

class Model:
	"""The Model represents the original texture list,
		the WIP texture list, and various bits of state.
	"""
	def parse_texture_file(self,fname):
		texture1 = file(fname, "r").read()
		textures = {}
		current = None
		for line in texture1.split("\n"):
			if len(line) == 0 or line[0] == ";" or line[0] == "#":
				continue
			elif line[0] == "*" and current:
				junk,name,y,x= line.split()
				current.patches.append(Patch(name,int(x),int(y)))
			else:
				line = line.split()
				current = Texture(line[0],line[1],line[2])
				textures[line[0]] = current
		return textures

	def new_wip_textures(self):
		for texture in self.orig_textures.values():
			self.wip_textures[texture.name] = \
				Texture(texture.name, texture.width, texture.height)

	def __init__(self):
		self.orig_textures = \
			self.parse_texture_file("../../textures/combined.txt")
		self.wip_textures = {}

model = Model()
print "\n\n\torig\n\n"
print "".join(map(str, model.orig_textures.values()))

print "\n\n\tnew\n\n"
model.new_wip_textures()
print "".join(map(str, model.wip_textures.values()))



#!/usr/bin/env python
#
# Wadinfo builder, rewritten in Python :)
#
# Copyright (c) 2006 Simon Howard.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   * Redistributions of source code must retain the above copyright 
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of Simon Howard nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import glob
import os
import sys
import re

dummy = False

# Given a file name, get the resource name for it:

def resource_for_filename(filename):
	if "." in filename:
		return filename[0:filename.index(".")]
	else:
		return filename

# Build look-up table, mapping section and resource name to filename.

def build_lookup_table():
	result = {}

	for filename in glob.glob("*/*"):
		section = os.path.dirname(filename)
		basename = os.path.basename(filename)

		resource = resource_for_filename(basename)

		result[(section, resource)] = filename

	return result

# Given a file base name, eg. "HEADE5", find the file to be used for
# that resource.  If the file cannot be found, None is returned.

def find_file(section, name):
	key = (section, name)

	if key in file_lut:
		return file_lut[key]
	else:
		return None

# Warning header displayed at the top of an output file.

def print_warning_header():
	print("; This file is automatically generated.")
	print("; Do not edit it directly!")
	print("")

# Remove comments beginning with "#" or ";"

def remove_comments(text):
	if "#" in text:
		return text[0:text.index("#")]
	elif ";" in text:
		return text[0:text.index(";")]
	else:
		return text

# Get the name of a dummy lump to use as a stand-in for the 
# given resource.

def get_dummy_name(resource):
	if resource.lower().startswith("demo"):
		return "fakedemo"
	else:
		return  "dummy"

# Parse an assignment statement.

def parse_assignment(section, line, match):
	resource = match.group(1).lower()
	override = match.group(3)

	# allow "= filename.ext" to override the filename used

	if override is not None:
		filename = find_file(section, override.lower())
	else:
		filename = find_file(section, resource)

	# File not found?

	if filename is None:

		# This resource hasn't been submitted yet, so either
		# comment the line out, or use a dummy resource,
		# depending on configuration.

		if dummy:
			dummy_name = get_dummy_name(resource)

			result = "%s = %s" % (resource, dummy_name)
		else:
			result = "; " + line
	else:
		# Resource found.

		result = line

	return result

# Parse data from the given input stream.

def parse_stream(stream):

	section_re = re.compile(r'\[(.*)\]')
	assignment_re = re.compile(r'\s*(\S+)[^\=]*(\s*\=\s*(\S+))?')

	print_warning_header()

	section = None

	# Parse each line of the input file, possibly changing things as we go.

	for line in stream:

		# Strip newline

		line = line[0:len(line) - 1]

		# Remove comments

		line = remove_comments(line)

		# start of new section?

		match = section_re.search(line)

		if match:
			section = match.group(1)

			if section == "texture1" or section == "texture2":
				section = "textures"

		else:
			# Possibly comment out assignments.

			match = assignment_re.match(line)

			if match:
				line = parse_assignment(section, line, match)

		print(line)

# Parse command line options:

for arg in sys.argv:
	if arg == "-dummy":
		dummy = True

# Build look-up table for files:

file_lut = build_lookup_table()

# Parse the input stream:

parse_stream(sys.stdin)




#!/usr/bin/python

from doom import Patch, Texture
import sys,re

if len(sys.argv) != 2:
	sys.stderr.write("usage: sw1_sw2.py <infile>\n")
	sys.exit()

infile = sys.argv[1]

# TODO: a generalized form of this para should probably be moved into the
# Texture class
texture1 = file(infile, "r").read()
textures = {}
current = None
for line in texture1.split("\n"):
	if len(line) == 0 or line[0] == ";" or line[0] == "#":
		continue
	elif line[0] == "*" and current:
		junk,name,y,x= line.split()
		current.patches.append(Patch(name,int(x),int(y)))
	else:
		line = line.split()
		current = Texture(line[0],line[1],line[2])
		textures[line[0]] = current

# a list of known switch textures
switches = {
	"SW1S0": "SW1S1",
	"SW2_1": "SW2_2",
	"SW2_3": "SW2_4",
	"SW2_5": "SW2_6",
	"SW2_7": "SW2_8",
	"SW2S0": "SW2S1",
	"SW3S0": "SW3S1",
	"SW4S0": "SW4S1"
}
# include the opposites
switches.update(dict([ (v,k) for k,v in switches.items() ]))

for sw1 in [x for x in textures.values() if re.match("^SW1", x.name)]:
	sw2n = sw1.name.replace("SW1","SW2")
	if not textures.has_key(sw2n):
		sys.stderr.write("error: %s not defined, skipping\n" % sw2n)
		continue
	sw2 = textures[sw2n]
	if len(sw2.patches) != 0:
		sys.stderr.write("error: %s has patches (%s)\n" % \
			(sw2.name, map(lambda x: x.name,sw2.patches)))
		continue

	# now the magic happens
	for patch in sw1.patches:
		name = patch.name
		if switches.has_key(patch.name):
			name = switches[patch.name]
		sw2.patches.append(Patch(name, patch.yoff,patch.xoff))
	textures[sw2n] = sw2

a = textures.values()
a.sort(lambda a,b: cmp(a.name,b.name))
sys.stdout.write(''.join(map(str,a)))



#!/usr/bin/env python
# 
# perform sanity check on assignments lists; make sure everything in
# the deutex tree is listed in the assignment lists
#
# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
# Contributors to the Freedoom project.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the freedoom project nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import glob
import re
import sys

# Find the file matching the given lump name:

def find_file(dirname, lumpname):

	if dirname == "musics":
		lumpname = "d_" + lumpname

	files = glob.glob("%s/%s*" % (dirname, lumpname))

	if len(files) > 0:
		return files[0]
	else:
		return None

# Perform sanity check on the given directory:

def sanity_check(dirname, filename):

	print "Checking %s" % dirname

	f = file(filename)

	for line in f:

		# Strip newline

		line = line[0:len(line)-1]

		# Ignore comments and empty lines

		if line == " " * len(line) or line[0] == "#":
			continue

		match = re.match(r'(\w+)', line)

		resource = match.group(1)

		# Find the file for this lump, if it exists

		filename = find_file(dirname, resource)

		# Is this resource supposed to be present? Check this
		# matches what is present.

		if re.search(r"(done|in dev)", line):
			if filename is None:
				print "%s: files not found for %s" % \
					(dirname, resource)
		else:
			if filename is None:
				print "%s: files found for %s" % \
					(dirname, resource)

	f.close()

status_dir = "status"

sections = {
	"sprites"  : "sprites_list",
	"patches"  : "patches_list",
	"flats"    : "flats_list",
	"graphics" : "graphics_list",
	"sounds"   : "sounds_list",
	"musics"   : "musics_list",
}

for section in sections.keys():
	sanity_check(section, "%s/%s" % (status_dir, sections[section]))




#!/usr/bin/env python
#
# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
# Contributors to the Freedoom project.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the freedoom project nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import sys
import re
import glob

texture_re = re.compile(r'\*\s+(\w+)')

# Parse a TEXTURE1 file and generate a list of patches

def parse_texture_file():
	patches = {}

	for line in sys.stdin:
		match = texture_re.match(line)

		if match:
			name = match.group(1)

			patches[name] = True

	print("; autogenerated patch list\n")

	for name in sorted(patches.keys()):
		print(name)

# Generate a full list of textures from the files in the
# patches/ directory

def list_all_textures():
	print("; autogenerated patch list\n")

	for filename in sorted(glob.glob("patches/*.gif")):
		base = os.path.basename(filename)
		patch_name = base[0:-4]
		print(patch_name)

if len(sys.argv) == 1:
	parse_texture_file()
elif sys.argv[1] == "-a":
	list_all_textures()
else:
	print("Usage: extract-pnames.py [-a]")

